function _awaitAsyncGenerator(value) { return new _AwaitValue(value); }

function _wrapAsyncGenerator(fn) { return function () { return new _AsyncGenerator(fn.apply(this, arguments)); }; }

function _AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; var wrappedAwait = value instanceof _AwaitValue; Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) { if (wrappedAwait) { resume(key === "return" ? "return" : "next", arg); return; } settle(result.done ? "return" : "normal", arg); }, function (err) { resume("throw", err); }); } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } }

if (typeof Symbol === "function" && Symbol.asyncIterator) { _AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; }

_AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); };

_AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); };

_AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); };

function _AwaitValue(value) { this.wrapped = value; }

import { Readable } from 'stream';
import { BATCH_SIZE, MAINTENANCE_MODE_AUTO, MAINTENANCE_MODE_MANUAL } from './constants';
import getNextCursor from './get-next-cursor';
import { AlreadyFrozenError, AlreadyUnfrozenError } from './frozen-errors';

async function startProcessEvents({
  pool,
  maintenanceMode
}) {
  if (maintenanceMode === MAINTENANCE_MODE_AUTO) {
    try {
      await pool.freeze();
    } catch (error) {
      if (!AlreadyFrozenError.is(error)) {
        throw error;
      }
    }
  }
}

async function endProcessEvents({
  pool,
  maintenanceMode
}) {
  if (maintenanceMode === MAINTENANCE_MODE_AUTO) {
    try {
      await pool.unfreeze();
    } catch (error) {
      if (!AlreadyUnfrozenError.is(error)) {
        throw error;
      }
    }
  }
}

function generator(_x) {
  return _generator.apply(this, arguments);
}

function _generator() {
  _generator = _wrapAsyncGenerator(function* (context) {
    const {
      pool,
      bufferSize
    } = context;
    yield _awaitAsyncGenerator(pool.waitConnect());
    yield _awaitAsyncGenerator(startProcessEvents(context));
    let eventsByteSize = 0;

    while (true) {
      const {
        events
      } = yield _awaitAsyncGenerator(pool.loadEventsByCursor({
        cursor: context.cursor,
        limit: BATCH_SIZE
      }));

      for (const event of events) {
        const chunk = Buffer.from(JSON.stringify(event) + '\n', 'utf8');
        const byteLength = chunk.byteLength;

        if (eventsByteSize + byteLength > bufferSize) {
          context.isBufferOverflow = true;
          yield _awaitAsyncGenerator(endProcessEvents(context));
          return;
        }

        eventsByteSize += byteLength;
        yield chunk;
        context.cursor = getNextCursor(context.cursor, [event]);
      }

      if (events.length === 0) {
        context.isEnd = true;
        yield _awaitAsyncGenerator(endProcessEvents(context));
        return;
      } else if (context.externalTimeout) {
        yield _awaitAsyncGenerator(endProcessEvents(context));
        return;
      }
    }
  });
  return _generator.apply(this, arguments);
}

const exportEventsStream = (pool, {
  cursor = null,
  maintenanceMode = MAINTENANCE_MODE_AUTO,
  bufferSize = Number.POSITIVE_INFINITY
} = {}) => {
  if (![MAINTENANCE_MODE_AUTO, MAINTENANCE_MODE_MANUAL].includes(maintenanceMode)) {
    throw new Error(`Wrong maintenance mode ${String(maintenanceMode)}`);
  }

  const context = {
    pool,
    maintenanceMode,
    cursor,
    bufferSize,
    isBufferOverflow: false,
    isEnd: false,
    externalTimeout: false
  };
  const stream = Readable.from(generator(context));
  stream.on('timeout', () => {
    context.externalTimeout = true;
  });
  Object.defineProperty(stream, 'cursor', {
    get() {
      return context.cursor;
    }

  });
  Object.defineProperty(stream, 'isBufferOverflow', {
    get() {
      return context.isBufferOverflow;
    }

  });
  Object.defineProperty(stream, 'isEnd', {
    get() {
      return context.isEnd;
    }

  });
  return stream;
};

export default exportEventsStream;
//# sourceMappingURL=export-events.js.map