/// <reference types="node" />
import { SecretsManager, Event, SerializableMap } from '@resolve-js/core';
import stream from 'stream';
import { MAINTENANCE_MODE_AUTO, MAINTENANCE_MODE_MANUAL } from './constants';
import * as t from 'io-ts';
export declare function validate<T extends t.Type<any>>(schema: T, params: any, errorName?: string): t.TypeOf<T>;
export declare type UnbrandProps<T extends any> = {
    [Property in keyof T]: T[Property] extends t.Branded<infer S, infer B> | infer Union ? S | Union : T[Property];
};
export declare type InputEvent = Event;
export declare type SavedEvent = Event & {
    threadCounter: number;
    threadId: number;
} & SerializableMap;
export declare type CheckForResourceError = (errors: Error[]) => void;
declare type DeleteSecret = SecretsManager['deleteSecret'];
declare type GetSecret = SecretsManager['getSecret'];
declare type SetSecret = SecretsManager['setSecret'];
declare type ShapeEvent = (event: any, additionalFields?: any) => SavedEvent;
export declare type ValidateEventFilter = (filter: EventFilter) => void;
export declare type GetNextCursor = (prevCursor: string | null, events: SavedEvent[]) => string;
export declare type EventsWithCursor = {
    cursor: string | null;
    events: SavedEvent[];
};
declare const TimestampFilterSchema: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.PartialC<{
    startTime: t.BrandC<t.NumberC, t.IntBrand>;
    finishTime: t.BrandC<t.NumberC, t.IntBrand>;
}>]>;
declare type TimestampFilterChecked = t.TypeOf<typeof TimestampFilterSchema>;
export declare type TimestampFilter = UnbrandProps<TimestampFilterChecked>;
declare const CursorFilterSchema: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.TypeC<{
    cursor: t.UnionC<[t.StringC, t.NullC]>;
}>]>;
declare type CursorFilterChecked = t.TypeOf<typeof CursorFilterSchema>;
export declare type CursorFilter = UnbrandProps<CursorFilterChecked>;
declare const EventFilterSchemaSimple: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.PartialC<{
    startTime: t.BrandC<t.NumberC, t.IntBrand>;
    finishTime: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    cursor: t.UnionC<[t.StringC, t.NullC]>;
}>]>;
export declare const EventFilterSchema: t.Type<UnbrandProps<{
    limit: t.Branded<number, t.IntBrand>;
} & {
    eventTypes?: string[] | null | undefined;
    aggregateIds?: string[] | null | undefined;
    eventSizeLimit?: t.Branded<number, t.IntBrand> | undefined;
} & {
    startTime?: t.Branded<number, t.IntBrand> | undefined;
    finishTime?: t.Branded<number, t.IntBrand> | undefined;
} & {
    cursor?: string | null | undefined;
}>, UnbrandProps<{
    limit: t.Branded<number, t.IntBrand>;
} & {
    eventTypes?: string[] | null | undefined;
    aggregateIds?: string[] | null | undefined;
    eventSizeLimit?: t.Branded<number, t.IntBrand> | undefined;
} & {
    startTime?: t.Branded<number, t.IntBrand> | undefined;
    finishTime?: t.Branded<number, t.IntBrand> | undefined;
} & {
    cursor?: string | null | undefined;
}>, unknown>;
declare type EventFilterChecked = t.TypeOf<typeof EventFilterSchemaSimple>;
export declare type EventFilter = UnbrandProps<EventFilterChecked>;
export declare type SecretFilter = {
    idx?: number | null;
    limit: number;
};
export declare type SecretsWithIdx = {
    idx: number | null;
    secrets: SecretRecord[];
};
export declare type SecretRecord = {
    idx: number;
    id: string;
    secret: string;
};
export declare function isTimestampFilter(filter: EventFilter): filter is TimestampFilter;
export declare function isCursorFilter(filter: EventFilter): filter is CursorFilter;
export declare type RemoveFirstType<T extends any[]> = T extends [infer _, ...infer R] ? R : never;
export declare type PromiseResultType<T extends any> = T extends Promise<infer R> ? R : T;
export declare type PoolMethod<ConnectedProps extends AdapterPoolConnectedProps, M extends (...args: any) => any> = (pool: AdapterPoolConnected<ConnectedProps>, ...args: Parameters<M>) => ReturnType<M>;
export declare type UnconnectedPoolMethod<ConnectedProps extends AdapterPoolConnectedProps, M extends (...args: any) => any> = (pool: AdapterPoolPossiblyUnconnected<ConnectedProps>, ...args: Parameters<M>) => ReturnType<M>;
export declare const AdapterConfigSchema: t.PartialC<{
    snapshotBucketSize: t.BrandC<t.NumberC, t.IntBrand>;
}>;
declare type AdapterConfigChecked = t.TypeOf<typeof AdapterConfigSchema>;
export declare type AdapterConfig = UnbrandProps<AdapterConfigChecked>;
export declare type AdapterPoolPrimalProps = {
    disposed: boolean;
    validateEventFilter: ValidateEventFilter;
    isInitialized: boolean;
    connectPromiseResolve?: any;
    connectPromise?: Promise<any>;
    maybeThrowResourceError: CheckForResourceError;
    bucketSize: number;
    counters: Map<string, number>;
    getNextCursor: GetNextCursor;
};
export declare type AdapterPoolConnectedProps = Adapter & {
    injectEvent: (event: SavedEvent) => Promise<void>;
    injectSecret?: (secretRecord: SecretRecord) => Promise<void>;
    loadEventsByTimestamp: (filter: TimestampFilter) => Promise<EventsWithCursor>;
    loadEventsByCursor: (filter: CursorFilter) => Promise<EventsWithCursor>;
    deleteSecret: DeleteSecret;
    getSecret: GetSecret;
    setSecret: SetSecret;
    waitConnect: any;
    shapeEvent: ShapeEvent;
    initEvents: () => Promise<any[]>;
    initSecrets: () => Promise<any[]>;
    initFinal: () => Promise<any[]>;
    dropEvents: () => Promise<any[]>;
    dropSecrets: () => Promise<any[]>;
    dropFinal: () => Promise<any[]>;
    ensureEventSubscriber: (params: {
        applicationName: string;
        eventSubscriber: string;
        destination?: any;
        status?: any;
        updateOnly?: boolean;
    }) => Promise<boolean>;
    removeEventSubscriber: (params: {
        applicationName: string;
        eventSubscriber: string;
    }) => Promise<void>;
    getEventSubscribers: (params?: {
        applicationName?: string;
        eventSubscriber?: string;
    } | undefined) => Promise<Array<{
        applicationName: string;
        eventSubscriber: string;
        destination: any;
        status: any;
    }>>;
};
export declare type AdapterPoolPossiblyUnconnected<ConnectedProps extends AdapterPoolConnectedProps> = AdapterPoolPrimalProps & Partial<ConnectedProps>;
export declare type AdapterPoolConnected<ConnectedProps extends AdapterPoolConnectedProps> = AdapterPoolPrimalProps & ConnectedProps;
export declare type WrappedConnectOnDemandAndCall<ConnectedProps extends AdapterPoolConnectedProps, M extends (pool: AdapterPoolConnected<ConnectedProps>, ...args: any) => any> = (...args: RemoveFirstType<Parameters<M>>) => Promise<PromiseResultType<ReturnType<M>>>;
declare type IfEquals<T, U, Y = unknown, N = never> = (<G>() => G extends T ? 1 : 2) extends <G>() => G extends U ? 1 : 2 ? Y : N;
export declare type WrapMethod<ConnectedProps extends AdapterPoolConnectedProps> = <M extends undefined | ((pool: AdapterPoolConnected<ConnectedProps>, ...args: any) => any)>(pool: AdapterPoolPossiblyUnconnected<ConnectedProps>, method: M) => IfEquals<M, undefined, undefined, WrappedConnectOnDemandAndCall<ConnectedProps, Exclude<M, undefined>>>;
export declare type WrapDispose<ConnectedProps extends AdapterPoolConnectedProps> = (pool: AdapterPoolPossiblyUnconnected<ConnectedProps>, dispose: PoolMethod<ConnectedProps, Adapter['dispose']>) => () => Promise<void>;
export declare type MAINTENANCE_MODE = typeof MAINTENANCE_MODE_AUTO | typeof MAINTENANCE_MODE_MANUAL;
export declare type ImportOptions = {
    byteOffset: number;
    maintenanceMode: MAINTENANCE_MODE;
};
export declare type ImportEventsStream = stream.Writable & {
    readonly byteOffset: number;
    readonly savedEventsCount: number;
};
export declare type ExportOptions = {
    cursor: string | null;
    maintenanceMode: MAINTENANCE_MODE;
    bufferSize: number;
};
export declare type ExportEventsStream = stream.Readable & {
    readonly cursor: string | null;
    readonly isBufferOverflow: boolean;
    readonly isEnd: boolean;
};
export declare type ImportSecretsOptions = {
    maintenanceMode: MAINTENANCE_MODE;
};
export declare type ExportSecretsOptions = {
    idx: number | null;
    maintenanceMode: MAINTENANCE_MODE;
};
export interface CommonAdapterFunctions<ConnectedProps extends AdapterPoolConnectedProps> {
    maybeThrowResourceError: CheckForResourceError;
    wrapMethod: WrapMethod<ConnectedProps>;
    wrapEventFilter: (loadEvents: PoolMethod<ConnectedProps, Adapter['loadEvents']>) => PoolMethod<ConnectedProps, Adapter['loadEvents']>;
    wrapDispose: WrapDispose<ConnectedProps>;
    validateEventFilter: ValidateEventFilter;
    loadEvents: PoolMethod<ConnectedProps, Adapter['loadEvents']>;
    importEventsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['importEvents']>;
    exportEventsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['exportEvents']>;
    incrementalImport: PoolMethod<ConnectedProps, Adapter['incrementalImport']>;
    getNextCursor: GetNextCursor;
    importSecretsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['importSecrets']>;
    exportSecretsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['exportSecrets']>;
    init: PoolMethod<ConnectedProps, Adapter['init']>;
    drop: PoolMethod<ConnectedProps, Adapter['drop']>;
}
export interface AdapterFunctions<ConnectedProps extends AdapterPoolConnectedProps, ConnectionDependencies extends any, Config extends AdapterConfig> {
    beginIncrementalImport: PoolMethod<ConnectedProps, Adapter['beginIncrementalImport']>;
    commitIncrementalImport: PoolMethod<ConnectedProps, Adapter['commitIncrementalImport']>;
    connect: (pool: AdapterPoolPossiblyUnconnected<ConnectedProps>, connectionDependencies: ConnectionDependencies, config: Config) => Promise<void>;
    dispose: PoolMethod<ConnectedProps, Adapter['dispose']>;
    dropSnapshot: PoolMethod<ConnectedProps, Adapter['dropSnapshot']>;
    freeze: PoolMethod<ConnectedProps, Adapter['freeze']>;
    getLatestEvent: PoolMethod<ConnectedProps, Adapter['getLatestEvent']>;
    injectEvent: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['injectEvent']>;
    loadEventsByCursor: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['loadEventsByCursor']>;
    loadEventsByTimestamp: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['loadEventsByTimestamp']>;
    loadSnapshot: PoolMethod<ConnectedProps, Adapter['loadSnapshot']>;
    pushIncrementalImport: PoolMethod<ConnectedProps, Adapter['pushIncrementalImport']>;
    rollbackIncrementalImport: PoolMethod<ConnectedProps, Adapter['rollbackIncrementalImport']>;
    saveEvent: PoolMethod<ConnectedProps, Adapter['saveEvent']>;
    saveSnapshot: PoolMethod<ConnectedProps, Adapter['saveSnapshot']>;
    shapeEvent: ShapeEvent;
    unfreeze: PoolMethod<ConnectedProps, Adapter['unfreeze']>;
    getSecret: PoolMethod<ConnectedProps, GetSecret>;
    setSecret: PoolMethod<ConnectedProps, SetSecret>;
    deleteSecret: PoolMethod<ConnectedProps, DeleteSecret>;
    loadSecrets?: PoolMethod<ConnectedProps, NonNullable<Adapter['loadSecrets']>>;
    injectSecret?: PoolMethod<ConnectedProps, NonNullable<AdapterPoolConnectedProps['injectSecret']>>;
    initEvents: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initEvents']>;
    initSecrets: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initSecrets']>;
    initFinal: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initFinal']>;
    dropEvents: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropEvents']>;
    dropSecrets: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropSecrets']>;
    dropFinal: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropFinal']>;
    ensureEventSubscriber: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['ensureEventSubscriber']>;
    removeEventSubscriber: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['removeEventSubscriber']>;
    getEventSubscribers: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['getEventSubscribers']>;
}
export interface Adapter {
    loadEvents: (filter: EventFilter) => Promise<EventsWithCursor>;
    importEvents: (options?: Partial<ImportOptions>) => ImportEventsStream;
    exportEvents: (options?: Partial<ExportOptions>) => ExportEventsStream;
    getLatestEvent: (filter: EventFilter) => Promise<SavedEvent | null>;
    saveEvent: (event: InputEvent) => Promise<void>;
    init: () => Promise<void>;
    drop: () => Promise<void>;
    dispose: () => Promise<void>;
    freeze: () => Promise<void>;
    unfreeze: () => Promise<void>;
    getNextCursor: (prevCursor: string | null, events: any[]) => string;
    getSecretsManager: () => Promise<SecretsManager>;
    loadSnapshot: (snapshotKey: string) => Promise<string | null>;
    saveSnapshot: (snapshotKey: string, content: string) => Promise<void>;
    dropSnapshot: (snapshotKey: string) => Promise<void>;
    pushIncrementalImport: (events: InputEvent[], importId: string) => Promise<void>;
    beginIncrementalImport: () => Promise<string>;
    commitIncrementalImport: (importId: string, validateAfterCommit?: any) => Promise<void>;
    rollbackIncrementalImport: () => Promise<void>;
    incrementalImport: (events: InputEvent[]) => Promise<void>;
    loadSecrets?: (filter: SecretFilter) => Promise<SecretsWithIdx>;
    importSecrets: (options?: Partial<ImportSecretsOptions>) => stream.Writable;
    exportSecrets: (options?: Partial<ExportSecretsOptions>) => stream.Readable;
    ensureEventSubscriber: (params: {
        applicationName: string;
        eventSubscriber: string;
        destination?: any;
        status?: any;
        updateOnly?: boolean;
    }) => Promise<boolean>;
    removeEventSubscriber: (params: {
        applicationName: string;
        eventSubscriber: string;
    }) => Promise<void>;
    getEventSubscribers: (params?: {
        applicationName?: string;
        eventSubscriber?: string;
    } | undefined) => Promise<Array<{
        applicationName: string;
        eventSubscriber: string;
        destination: any;
        status: any;
    }>>;
}
export {};
